@@ -5,9 +5,8 @@ package Test::Exception;
 use Test::Builder;
 use Sub::Uplevel qw( uplevel );
 use base qw( Exporter );
-use Carp;
 
-our $VERSION = '0.26';
+our $VERSION = '0.31';
 our @EXPORT = qw(dies_ok lives_ok throws_ok lives_and);
 
 my $Tester = Test::Builder->new;
@@ -83,9 +82,57 @@ will not catch this with any of its testing functions.
 
 =cut
 
+sub _quiet_caller (;$) { ## no critic Prototypes
+    my $height = $_[0];
+    $height++;
+
+    if ( CORE::caller() eq 'DB' ) {
+        # passthrough the @DB::args trick
+        package DB;
+        if( wantarray ) {
+            if ( !@_ ) {
+                return (CORE::caller($height))[0..2];
+            }
+            else {
+                # If we got here, we are within a Test::Exception test, and
+                # something is producing a stacktrace. In case this is a full
+                # trace (i.e. confess() ), we have to make sure that the sub
+                # args are not visible. If we do not do this, and the test in
+                # question is throws_ok() with a regex, it will end up matching
+                # against itself in the args to throws_ok().
+                #
+                # While it is possible (and maybe wise), to test if we are
+                # indeed running under throws_ok (by crawling the stack right
+                # up from here), the old behavior of Test::Exception was to
+                # simply obliterate @DB::args altogether in _quiet_caller, so
+                # we are just preserving the behavior to avoid surprises
+                #
+                my @frame_info = CORE::caller($height);
+                @DB::args = ();
+                return @frame_info;
+            }
+        }
+
+        # fallback if nothing above returns
+        return CORE::caller($height);
+    }
+    else {
+        if( wantarray and !@_ ) {
+            return (CORE::caller($height))[0..2];
+        }
+        else {
+            return CORE::caller($height);
+        }
+    }
+}
 
 sub _try_as_caller {
     my $coderef = shift;
+
+    # local works here because Sub::Uplevel has already overridden caller
+    local *CORE::GLOBAL::caller;
+    { no warnings 'redefine'; *CORE::GLOBAL::caller = \&_quiet_caller; }
+
     eval { uplevel 3, $coderef };
     return $@;
 };
@@ -151,13 +198,20 @@ A true value is returned if the test succeeds, false otherwise. On exit $@ is gu
 
 A description of the exception being checked is used if no optional test description is passed.
 
+NOTE: Rememeber when you C<die $string_without_a_trailing_newline> perl will 
+automatically add the current script line number, input line number and a newline. This will
+form part of the string that throws_ok regular expressions match against.
+
+
 =cut
 
 
 sub throws_ok (&$;$) {
     my ( $coderef, $expecting, $description ) = @_;
-    croak "throws_ok: must pass exception class/object or regex" 
-        unless defined $expecting;
+    unless (defined $expecting) {
+      require Carp;
+      Carp::croak( "throws_ok: must pass exception class/object or regex" ); 
+    }
     $description = _exception_as_string( "threw", $expecting )
     	unless defined $description;
     my $exception = _try_as_caller( $coderef );
@@ -378,7 +432,9 @@ chromatic,
 Curt Sampson,
 David Cantrell,
 David Golden, 
+David Tulloh,
 David Wheeler, 
+J. K. O'Brien,
 Janek Schleicher,
 Jim Keenan, 
 Jos I. Boumans, 
@@ -389,15 +445,16 @@ Michael G Schwern,
 Nadim Khemir,
 Paul McCann,
 Perrin Harkins, 
+Peter Rabbitson,
 Peter Scott, 
-Rob Muhlestein 
+Ricardo Signes,
+Rob Muhlestein,
 Scott R. Godin,
 Steve Purkis,
 Steve, 
 Tim Bunce,
 and various anonymous folk for comments, suggestions, bug reports and patches.
 
-
 =head1 AUTHOR
 
 Adrian Howard <adrianh@quietstars.com>
